@@ -74,18 +74,20 @@
 #define yychar          aagchar
 
 /* First part of user prologue.  */
-#line 19 "../../lib/cgraph/grammar.y"
+#line 23 "../../lib/cgraph/grammar.y"
 
 
 #include <stdbool.h>
 #include <stdio.h>
 #include <cghdr.h>
+#include <cgraph/agxbuf.h>
 #include <cgraph/alloc.h>
+#include <cgraph/streq.h>
 #include <cgraph/unreachable.h>
 #include <stddef.h>
 extern void aagerror(const char*);
 
-static char Key[] = "key";
+static const char Key[] = "key";
 static int SubgraphDepth = 0;
 
 typedef union s {					/* possible items in generic list */
@@ -142,7 +144,7 @@ static	Agdisc_t	*Disc;		/* discipline passed to agread or agconcat */
 static gstack_t *S;
 
 
-#line 146 "grammar.c"
+#line 148 "grammar.c"
 
 # ifndef YY_CAST
 #  ifdef __cplusplus
@@ -228,13 +230,13 @@ extern int aagdebug;
 #if ! defined AAGSTYPE && ! defined AAGSTYPE_IS_DECLARED
 union AAGSTYPE
 {
-#line 87 "../../lib/cgraph/grammar.y"
+#line 93 "../../lib/cgraph/grammar.y"
 
 			int				i;
 			char			*str;
 			struct Agnode_s	*n;
 
-#line 238 "grammar.c"
+#line 240 "grammar.c"
 
 };
 typedef union AAGSTYPE AAGSTYPE;
@@ -610,12 +612,12 @@ static const yytype_int8 yytranslate[] =
   /* YYRLINE[YYN] -- Source line where rule number YYN was defined.  */
 static const yytype_uint8 yyrline[] =
 {
-       0,   104,   104,   105,   106,   109,   111,   114,   114,   116,
-     116,   118,   118,   120,   120,   122,   122,   124,   124,   126,
-     127,   130,   134,   134,   136,   136,   136,   137,   141,   141,
-     143,   144,   145,   148,   149,   152,   153,   154,   157,   158,
-     161,   161,   163,   165,   166,   168,   171,   174,   177,   177,
-     180,   181,   182,   185,   185,   185,   187,   188,   191,   192
+       0,   110,   110,   111,   112,   115,   117,   120,   120,   122,
+     122,   124,   124,   126,   126,   128,   128,   130,   130,   132,
+     133,   136,   140,   140,   142,   142,   142,   143,   147,   147,
+     149,   150,   151,   154,   155,   158,   159,   160,   163,   164,
+     167,   167,   169,   171,   172,   174,   177,   180,   183,   183,
+     186,   187,   188,   191,   191,   191,   193,   194,   197,   198
 };
 #endif
 
@@ -1456,211 +1458,211 @@ yyreduce:
   switch (yyn)
     {
   case 2:
-#line 104 "../../lib/cgraph/grammar.y"
+#line 110 "../../lib/cgraph/grammar.y"
                             {freestack(); endgraph();}
-#line 1462 "grammar.c"
+#line 1464 "grammar.c"
     break;
 
   case 3:
-#line 105 "../../lib/cgraph/grammar.y"
+#line 111 "../../lib/cgraph/grammar.y"
                                         {if (G) {freestack(); endgraph(); agclose(G); G = Ag_G_global = NULL;}}
-#line 1468 "grammar.c"
+#line 1470 "grammar.c"
     break;
 
   case 6:
-#line 111 "../../lib/cgraph/grammar.y"
+#line 117 "../../lib/cgraph/grammar.y"
                                                                  {startgraph((yyvsp[0].str),(yyvsp[-1].i) != 0,(yyvsp[-2].i) != 0);}
-#line 1474 "grammar.c"
+#line 1476 "grammar.c"
     break;
 
   case 7:
-#line 114 "../../lib/cgraph/grammar.y"
+#line 120 "../../lib/cgraph/grammar.y"
                      {(yyval.str)=(yyvsp[0].str);}
-#line 1480 "grammar.c"
+#line 1482 "grammar.c"
     break;
 
   case 8:
-#line 114 "../../lib/cgraph/grammar.y"
+#line 120 "../../lib/cgraph/grammar.y"
                                             {(yyval.str)=0;}
-#line 1486 "grammar.c"
+#line 1488 "grammar.c"
     break;
 
   case 9:
-#line 116 "../../lib/cgraph/grammar.y"
+#line 122 "../../lib/cgraph/grammar.y"
                              {(yyval.i)=1;}
-#line 1492 "grammar.c"
+#line 1494 "grammar.c"
     break;
 
   case 10:
-#line 116 "../../lib/cgraph/grammar.y"
+#line 122 "../../lib/cgraph/grammar.y"
                                                     {(yyval.i)=0;}
-#line 1498 "grammar.c"
+#line 1500 "grammar.c"
     break;
 
   case 11:
-#line 118 "../../lib/cgraph/grammar.y"
+#line 124 "../../lib/cgraph/grammar.y"
                                 {(yyval.i) = 0;}
-#line 1504 "grammar.c"
+#line 1506 "grammar.c"
     break;
 
   case 12:
-#line 118 "../../lib/cgraph/grammar.y"
+#line 124 "../../lib/cgraph/grammar.y"
                                                           {(yyval.i) = 1;}
-#line 1510 "grammar.c"
+#line 1512 "grammar.c"
     break;
 
   case 21:
-#line 131 "../../lib/cgraph/grammar.y"
+#line 137 "../../lib/cgraph/grammar.y"
                                         {if ((yyvsp[-1].i)) endedge(); else endnode();}
-#line 1516 "grammar.c"
+#line 1518 "grammar.c"
     break;
 
   case 24:
-#line 136 "../../lib/cgraph/grammar.y"
+#line 142 "../../lib/cgraph/grammar.y"
                                  {getedgeitems();}
-#line 1522 "grammar.c"
+#line 1524 "grammar.c"
     break;
 
   case 25:
-#line 136 "../../lib/cgraph/grammar.y"
+#line 142 "../../lib/cgraph/grammar.y"
                                                           {getedgeitems();}
-#line 1528 "grammar.c"
+#line 1530 "grammar.c"
     break;
 
   case 26:
-#line 136 "../../lib/cgraph/grammar.y"
+#line 142 "../../lib/cgraph/grammar.y"
                                                                                       {(yyval.i) = 1;}
-#line 1534 "grammar.c"
+#line 1536 "grammar.c"
     break;
 
   case 27:
-#line 137 "../../lib/cgraph/grammar.y"
+#line 143 "../../lib/cgraph/grammar.y"
                                             {(yyval.i) = 0;}
-#line 1540 "grammar.c"
+#line 1542 "grammar.c"
     break;
 
   case 30:
-#line 143 "../../lib/cgraph/grammar.y"
+#line 149 "../../lib/cgraph/grammar.y"
                        {appendnode((yyvsp[0].str),NULL,NULL);}
-#line 1546 "grammar.c"
+#line 1548 "grammar.c"
     break;
 
   case 31:
-#line 144 "../../lib/cgraph/grammar.y"
+#line 150 "../../lib/cgraph/grammar.y"
                             {appendnode((yyvsp[-2].str),(yyvsp[0].str),NULL);}
-#line 1552 "grammar.c"
+#line 1554 "grammar.c"
     break;
 
   case 32:
-#line 145 "../../lib/cgraph/grammar.y"
+#line 151 "../../lib/cgraph/grammar.y"
                                      {appendnode((yyvsp[-4].str),(yyvsp[-2].str),(yyvsp[0].str));}
-#line 1558 "grammar.c"
+#line 1560 "grammar.c"
     break;
 
   case 33:
-#line 148 "../../lib/cgraph/grammar.y"
+#line 154 "../../lib/cgraph/grammar.y"
                                                   {attrstmt((yyvsp[-2].i),(yyvsp[-1].str));}
-#line 1564 "grammar.c"
+#line 1566 "grammar.c"
     break;
 
   case 34:
-#line 149 "../../lib/cgraph/grammar.y"
+#line 155 "../../lib/cgraph/grammar.y"
                                          {attrstmt(T_graph,NULL);}
-#line 1570 "grammar.c"
+#line 1572 "grammar.c"
     break;
 
   case 35:
-#line 152 "../../lib/cgraph/grammar.y"
+#line 158 "../../lib/cgraph/grammar.y"
                         {(yyval.i) = T_graph;}
-#line 1576 "grammar.c"
+#line 1578 "grammar.c"
     break;
 
   case 36:
-#line 153 "../../lib/cgraph/grammar.y"
+#line 159 "../../lib/cgraph/grammar.y"
                                  {(yyval.i) = T_node;}
-#line 1582 "grammar.c"
+#line 1584 "grammar.c"
     break;
 
   case 37:
-#line 154 "../../lib/cgraph/grammar.y"
+#line 160 "../../lib/cgraph/grammar.y"
                                  {(yyval.i) = T_edge;}
-#line 1588 "grammar.c"
+#line 1590 "grammar.c"
     break;
 
   case 38:
-#line 157 "../../lib/cgraph/grammar.y"
+#line 163 "../../lib/cgraph/grammar.y"
                         {(yyval.str) = (yyvsp[-1].str);}
-#line 1594 "grammar.c"
+#line 1596 "grammar.c"
     break;
 
   case 39:
-#line 158 "../../lib/cgraph/grammar.y"
+#line 164 "../../lib/cgraph/grammar.y"
                                       {(yyval.str) = NULL; }
-#line 1600 "grammar.c"
+#line 1602 "grammar.c"
     break;
 
   case 46:
-#line 171 "../../lib/cgraph/grammar.y"
+#line 177 "../../lib/cgraph/grammar.y"
                                  {appendattr((yyvsp[-2].str),(yyvsp[0].str));}
-#line 1606 "grammar.c"
+#line 1608 "grammar.c"
     break;
 
   case 48:
-#line 177 "../../lib/cgraph/grammar.y"
+#line 183 "../../lib/cgraph/grammar.y"
                               {opensubg((yyvsp[0].str));}
-#line 1612 "grammar.c"
+#line 1614 "grammar.c"
     break;
 
   case 49:
-#line 177 "../../lib/cgraph/grammar.y"
+#line 183 "../../lib/cgraph/grammar.y"
                                                    {closesubg();}
-#line 1618 "grammar.c"
+#line 1620 "grammar.c"
     break;
 
   case 50:
-#line 180 "../../lib/cgraph/grammar.y"
+#line 186 "../../lib/cgraph/grammar.y"
                                   {(yyval.str)=(yyvsp[0].str);}
-#line 1624 "grammar.c"
+#line 1626 "grammar.c"
     break;
 
   case 51:
-#line 181 "../../lib/cgraph/grammar.y"
+#line 187 "../../lib/cgraph/grammar.y"
                                       {(yyval.str)=NULL;}
-#line 1630 "grammar.c"
+#line 1632 "grammar.c"
     break;
 
   case 52:
-#line 182 "../../lib/cgraph/grammar.y"
+#line 188 "../../lib/cgraph/grammar.y"
                                       {(yyval.str)=NULL;}
-#line 1636 "grammar.c"
+#line 1638 "grammar.c"
     break;
 
   case 56:
-#line 187 "../../lib/cgraph/grammar.y"
+#line 193 "../../lib/cgraph/grammar.y"
                   {(yyval.str) = (yyvsp[0].str);}
-#line 1642 "grammar.c"
+#line 1644 "grammar.c"
     break;
 
   case 57:
-#line 188 "../../lib/cgraph/grammar.y"
+#line 194 "../../lib/cgraph/grammar.y"
                                  {(yyval.str) = (yyvsp[0].str);}
-#line 1648 "grammar.c"
+#line 1650 "grammar.c"
     break;
 
   case 58:
-#line 191 "../../lib/cgraph/grammar.y"
+#line 197 "../../lib/cgraph/grammar.y"
                    {(yyval.str) = (yyvsp[0].str);}
-#line 1654 "grammar.c"
+#line 1656 "grammar.c"
     break;
 
   case 59:
-#line 192 "../../lib/cgraph/grammar.y"
+#line 198 "../../lib/cgraph/grammar.y"
                                              {(yyval.str) = concat((yyvsp[-2].str),(yyvsp[0].str));}
-#line 1660 "grammar.c"
+#line 1662 "grammar.c"
     break;
 
 
-#line 1664 "grammar.c"
+#line 1666 "grammar.c"
 
       default: break;
     }
@@ -1892,7 +1894,7 @@ yyreturn:
 #endif
   return yyresult;
 }
-#line 194 "../../lib/cgraph/grammar.y"
+#line 200 "../../lib/cgraph/grammar.y"
 
 
 static item *newitem(int tag, void *p0, char *p1)
@@ -2003,7 +2005,7 @@ static void applyattrs(void *obj)
 
 static void nomacros(void)
 {
-	agerr(AGWARN,"attribute macros not implemented");
+  agwarningf("attribute macros not implemented");
 }
 
 /* attrstmt:
@@ -2037,7 +2039,7 @@ static void attrstmt(int tkind, char *macroname)
 		else
 			sym = aptr->u.asym;
 		if (S->g == G)
-			sym->print = TRUE;
+			sym->print = true;
 	}
 	deletelist(&(S->attrlist));
 }
@@ -2051,7 +2053,7 @@ static void appendnode(char *name, char *port, char *sport)
 	if (sport) {
 		port = concatPort (port, sport);
 	}
-	elt = cons_node(agnode(S->g,name,TRUE),port);
+	elt = cons_node(agnode(S->g, name, 1), port);
 	listapp(&(S->nodelist),elt);
 	agstrfree(G,name);
 }
@@ -2145,23 +2147,16 @@ concat (char* s1, char* s2)
   return s;
 }
 
-/* concatPort:
- */
 static char*
 concatPort (char* s1, char* s2)
 {
-  char*  s;
-  char   buf[BUFSIZ];
-  char*  sym;
-  size_t len = strlen(s1) + strlen(s2) + 2;  /* one more for ':' */
+  agxbuf buf = {0};
 
-  if (len <= BUFSIZ) sym = buf;
-  else sym = gv_alloc(len);
-  sprintf (sym, "%s:%s", s1, s2);
-  s = agstrdup (G,sym);
+  agxbprint(&buf, "%s:%s", s1, s2);
+  char *s = agstrdup(G, agxbuse(&buf));
   agstrfree (G,s1);
   agstrfree (G,s2);
-  if (sym != buf) free (sym);
+  agxbfree(&buf);
   return s;
 }
 
@@ -2197,7 +2192,7 @@ static void newedge(Agnode_t *t, char *tport, Agnode_t *h, char *hport, char *ke
 {
 	Agedge_t 	*e;
 
-	e = agedge(S->g,t,h,key,TRUE);
+	e = agedge(S->g, t, h, key, 1);
 	if (e) {		/* can fail if graph is strict and t==h */
 		char    *tp = tport;
 		char    *hp = hport;
@@ -2238,9 +2233,9 @@ static void endgraph(void)
 static void opensubg(char *name)
 {
   if (++SubgraphDepth >= YYMAXDEPTH/2) {
-    agerr(AGERR,"subgraphs nested more than %d deep",YYMAXDEPTH);
+    agerrorf("subgraphs nested more than %d deep", YYMAXDEPTH);
   }
-	S = push(S,agsubg(S->g,name,TRUE));
+	S = push(S, agsubg(S->g, name, 1));
 	agstrfree(G,name);
 }
 
