@@ -74,7 +74,7 @@
 #define yychar          htmlchar
 
 /* First part of user prologue.  */
-#line 19 "../../lib/common/htmlparse.y"
+#line 21 "../../lib/common/htmlparse.y"
 
 
 #include <cgraph/alloc.h>
@@ -103,23 +103,12 @@ static struct {
  * Free row. This closes and frees row's list, then
  * the pitem itself is freed.
  */
-static void free_ritem(pitem *p, Dtdisc_t *ds) {
-  (void)ds;
-
+static void free_ritem(void *item) {
+  pitem *p = item;
   dtclose (p->u.rp);
   free (p);
 }
 
-/* free_item:
- * Generic Dt free. Only frees container, assuming contents
- * have been copied elsewhere.
- */
-static void free_item(void *p, Dtdisc_t *ds) {
-  (void)ds;
-
-  free (p);
-}
-
 /* cleanTbl:
  * Clean up table if error in parsing.
  */
@@ -146,9 +135,8 @@ cleanCell (htmlcell_t* cp)
 /* free_citem:
  * Free cell item during parsing. This frees cell and pitem.
  */
-static void free_citem(pitem *p, Dtdisc_t *ds) {
-  (void)ds;
-
+static void free_citem(void *item) {
+  pitem *p = item;
   cleanCell (p->u.cp);
   free (p);
 }
@@ -157,13 +145,13 @@ static Dtdisc_t rowDisc = {
     .key = offsetof(pitem, u),
     .size = sizeof(void *),
     .link = offsetof(pitem, link),
-    .freef = (Dtfree_f)free_ritem,
+    .freef = free_ritem,
 };
 static Dtdisc_t cellDisc = {
     .key = offsetof(pitem, u),
     .size = sizeof(void *),
     .link = offsetof(pitem, link),
-    .freef = (Dtfree_f)free_item,
+    .freef = free,
 };
 
 typedef struct {
@@ -176,16 +164,14 @@ typedef struct {
     htextspan_t  lp;
 } fspan;
 
-static void free_fitem(fitem *p, Dtdisc_t *ds) {
-    (void)ds;
-
+static void free_fitem(void *item) {
+    fitem *p = item;
     free (p->ti.str);
     free (p);
 }
 
-static void free_fspan(fspan *p, Dtdisc_t *ds) {
-    (void)ds;
-
+static void free_fspan(void *span) {
+    fspan *p = span;
     textspan_t* ti;
 
     if (p->lp.nitems) {
@@ -201,12 +187,12 @@ static void free_fspan(fspan *p, Dtdisc_t *ds) {
 
 static Dtdisc_t fstrDisc = {
     .link = offsetof(fitem, link),
-    .freef = (Dtfree_f)free_item,
+    .freef = free,
 };
 
 static Dtdisc_t fspanDisc = {
     .link = offsetof(fspan, link),
-    .freef = (Dtfree_f)free_item,
+    .freef = free,
 };
 
 /* appendFItemList:
@@ -236,11 +222,11 @@ appendFLineList (int v)
     if (cnt) {
         int i = 0;
 	ln->lp.nitems = cnt;
-	ln->lp.items = N_NEW(cnt, textspan_t);
+	ln->lp.items = gv_calloc(cnt, sizeof(textspan_t));
 
 	fi = (fitem*)dtflatten(ilist);
 	for (; fi; fi = (fitem*)dtlink(fitemList, fi)) {
-		/* NOTE: When fitemList is closed, it uses free_item, which only frees the container,
+		/* NOTE: When fitemList is closed, it uses free, which only frees the container,
 		 * not the contents, so this copy is safe.
 		 */
 	    ln->lp.items[i] = fi->ti;  
@@ -274,7 +260,7 @@ mkText(void)
     	
     if (cnt) {
 	int i = 0;
-	hft->spans = N_NEW(cnt,htextspan_t);	
+	hft->spans = gv_calloc(cnt, sizeof(htextspan_t));
     	for(fl=dtfirst(ispan); fl; fl=dtnext(ispan,fl)) {
     	    hft->spans[i] = fl->lp;
     	    i++;
@@ -374,21 +360,21 @@ static void cleanup (void)
     free_html_label (HTMLstate.lbl,1);
     HTMLstate.lbl = NULL;
   }
-  cellDisc.freef = (Dtfree_f)free_citem;
+  cellDisc.freef = free_citem;
   while (tp) {
     next = tp->u.p.prev;
     cleanTbl (tp);
     tp = next;
   }
-  cellDisc.freef = (Dtfree_f)free_item;
+  cellDisc.freef = free;
 
-  fstrDisc.freef = (Dtfree_f)free_fitem;
+  fstrDisc.freef = free_fitem;
   dtclear (HTMLstate.fitemList);
-  fstrDisc.freef = (Dtfree_f)free_item;
+  fstrDisc.freef = free;
 
-  fspanDisc.freef = (Dtfree_f)free_fspan;
+  fspanDisc.freef = free_fspan;
   dtclear (HTMLstate.fspanList);
-  fspanDisc.freef = (Dtfree_f)free_item;
+  fspanDisc.freef = free;
 
   freeFontstack();
 }
@@ -445,7 +431,7 @@ popFont (void)
 }
 
 
-#line 449 "htmlparse.c"
+#line 435 "htmlparse.c"
 
 # ifndef YY_CAST
 #  ifdef __cplusplus
@@ -585,7 +571,7 @@ extern int htmldebug;
 #if ! defined HTMLSTYPE && ! defined HTMLSTYPE_IS_DECLARED
 union HTMLSTYPE
 {
-#line 390 "../../lib/common/htmlparse.y"
+#line 378 "../../lib/common/htmlparse.y"
 
   int    i;
   htmltxt_t*  txt;
@@ -595,7 +581,7 @@ union HTMLSTYPE
   htmlimg_t*   img;
   pitem*       p;
 
-#line 599 "htmlparse.c"
+#line 585 "htmlparse.c"
 
 };
 typedef union HTMLSTYPE HTMLSTYPE;
@@ -974,13 +960,13 @@ static const yytype_int8 yytranslate[] =
   /* YYRLINE[YYN] -- Source line where rule number YYN was defined.  */
 static const yytype_int16 yyrline[] =
 {
-       0,   422,   422,   423,   424,   427,   430,   431,   434,   435,
-     436,   437,   438,   439,   440,   441,   442,   443,   446,   449,
-     452,   455,   458,   461,   464,   467,   470,   473,   476,   479,
-     482,   485,   488,   491,   494,   495,   498,   499,   502,   502,
-     523,   524,   525,   526,   527,   528,   531,   532,   535,   536,
-     537,   540,   540,   543,   544,   545,   548,   548,   549,   549,
-     550,   550,   551,   551,   554,   555,   558,   559,   562,   563
+       0,   410,   410,   411,   412,   415,   418,   419,   422,   423,
+     424,   425,   426,   427,   428,   429,   430,   431,   434,   437,
+     440,   443,   446,   449,   452,   455,   458,   461,   464,   467,
+     470,   473,   476,   479,   482,   483,   486,   487,   490,   490,
+     511,   512,   513,   514,   515,   516,   519,   520,   523,   524,
+     525,   528,   528,   531,   532,   533,   536,   536,   537,   537,
+     538,   538,   539,   539,   542,   543,   546,   547,   550,   551
 };
 #endif
 
@@ -1884,151 +1870,151 @@ yyreduce:
   switch (yyn)
     {
   case 2:
-#line 422 "../../lib/common/htmlparse.y"
+#line 410 "../../lib/common/htmlparse.y"
                                    { HTMLstate.lbl = mkLabel((yyvsp[-1].txt),HTML_TEXT); }
-#line 1890 "htmlparse.c"
+#line 1876 "htmlparse.c"
     break;
 
   case 3:
-#line 423 "../../lib/common/htmlparse.y"
+#line 411 "../../lib/common/htmlparse.y"
                                     { HTMLstate.lbl = mkLabel((yyvsp[-1].tbl),HTML_TBL); }
-#line 1896 "htmlparse.c"
+#line 1882 "htmlparse.c"
     break;
 
   case 4:
-#line 424 "../../lib/common/htmlparse.y"
+#line 412 "../../lib/common/htmlparse.y"
               { cleanup(); YYABORT; }
-#line 1902 "htmlparse.c"
+#line 1888 "htmlparse.c"
     break;
 
   case 5:
-#line 427 "../../lib/common/htmlparse.y"
+#line 415 "../../lib/common/htmlparse.y"
                 { (yyval.txt) = mkText(); }
-#line 1908 "htmlparse.c"
+#line 1894 "htmlparse.c"
     break;
 
   case 8:
-#line 434 "../../lib/common/htmlparse.y"
+#line 422 "../../lib/common/htmlparse.y"
                   { appendFItemList(HTMLstate.str);}
-#line 1914 "htmlparse.c"
+#line 1900 "htmlparse.c"
     break;
 
   case 9:
-#line 435 "../../lib/common/htmlparse.y"
+#line 423 "../../lib/common/htmlparse.y"
               {appendFLineList((yyvsp[0].i));}
-#line 1920 "htmlparse.c"
+#line 1906 "htmlparse.c"
     break;
 
   case 18:
-#line 446 "../../lib/common/htmlparse.y"
+#line 434 "../../lib/common/htmlparse.y"
               { pushFont ((yyvsp[0].font)); }
-#line 1926 "htmlparse.c"
+#line 1912 "htmlparse.c"
     break;
 
   case 19:
-#line 449 "../../lib/common/htmlparse.y"
+#line 437 "../../lib/common/htmlparse.y"
                     { popFont (); }
-#line 1932 "htmlparse.c"
+#line 1918 "htmlparse.c"
     break;
 
   case 20:
-#line 452 "../../lib/common/htmlparse.y"
+#line 440 "../../lib/common/htmlparse.y"
                   {pushFont((yyvsp[0].font));}
-#line 1938 "htmlparse.c"
+#line 1924 "htmlparse.c"
     break;
 
   case 21:
-#line 455 "../../lib/common/htmlparse.y"
+#line 443 "../../lib/common/htmlparse.y"
                       {popFont();}
-#line 1944 "htmlparse.c"
+#line 1930 "htmlparse.c"
     break;
 
   case 22:
-#line 458 "../../lib/common/htmlparse.y"
+#line 446 "../../lib/common/htmlparse.y"
               {pushFont((yyvsp[0].font));}
-#line 1950 "htmlparse.c"
+#line 1936 "htmlparse.c"
     break;
 
   case 23:
-#line 461 "../../lib/common/htmlparse.y"
+#line 449 "../../lib/common/htmlparse.y"
                   {popFont();}
-#line 1956 "htmlparse.c"
+#line 1942 "htmlparse.c"
     break;
 
   case 24:
-#line 464 "../../lib/common/htmlparse.y"
+#line 452 "../../lib/common/htmlparse.y"
              {pushFont((yyvsp[0].font));}
-#line 1962 "htmlparse.c"
+#line 1948 "htmlparse.c"
     break;
 
   case 25:
-#line 467 "../../lib/common/htmlparse.y"
+#line 455 "../../lib/common/htmlparse.y"
                  {popFont();}
-#line 1968 "htmlparse.c"
+#line 1954 "htmlparse.c"
     break;
 
   case 26:
-#line 470 "../../lib/common/htmlparse.y"
+#line 458 "../../lib/common/htmlparse.y"
                         {pushFont((yyvsp[0].font));}
-#line 1974 "htmlparse.c"
+#line 1960 "htmlparse.c"
     break;
 
   case 27:
-#line 473 "../../lib/common/htmlparse.y"
+#line 461 "../../lib/common/htmlparse.y"
                             {popFont();}
-#line 1980 "htmlparse.c"
+#line 1966 "htmlparse.c"
     break;
 
   case 28:
-#line 476 "../../lib/common/htmlparse.y"
+#line 464 "../../lib/common/htmlparse.y"
                       {pushFont((yyvsp[0].font));}
-#line 1986 "htmlparse.c"
+#line 1972 "htmlparse.c"
     break;
 
   case 29:
-#line 479 "../../lib/common/htmlparse.y"
+#line 467 "../../lib/common/htmlparse.y"
                           {popFont();}
-#line 1992 "htmlparse.c"
+#line 1978 "htmlparse.c"
     break;
 
   case 30:
-#line 482 "../../lib/common/htmlparse.y"
+#line 470 "../../lib/common/htmlparse.y"
             {pushFont((yyvsp[0].font));}
-#line 1998 "htmlparse.c"
+#line 1984 "htmlparse.c"
     break;
 
   case 31:
-#line 485 "../../lib/common/htmlparse.y"
+#line 473 "../../lib/common/htmlparse.y"
                 {popFont();}
-#line 2004 "htmlparse.c"
+#line 1990 "htmlparse.c"
     break;
 
   case 32:
-#line 488 "../../lib/common/htmlparse.y"
+#line 476 "../../lib/common/htmlparse.y"
             {pushFont((yyvsp[0].font));}
-#line 2010 "htmlparse.c"
+#line 1996 "htmlparse.c"
     break;
 
   case 33:
-#line 491 "../../lib/common/htmlparse.y"
+#line 479 "../../lib/common/htmlparse.y"
                 {popFont();}
-#line 2016 "htmlparse.c"
+#line 2002 "htmlparse.c"
     break;
 
   case 34:
-#line 494 "../../lib/common/htmlparse.y"
+#line 482 "../../lib/common/htmlparse.y"
                        { (yyval.i) = (yyvsp[-1].i); }
-#line 2022 "htmlparse.c"
+#line 2008 "htmlparse.c"
     break;
 
   case 35:
-#line 495 "../../lib/common/htmlparse.y"
+#line 483 "../../lib/common/htmlparse.y"
               { (yyval.i) = (yyvsp[0].i); }
-#line 2028 "htmlparse.c"
+#line 2014 "htmlparse.c"
     break;
 
   case 38:
-#line 502 "../../lib/common/htmlparse.y"
+#line 490 "../../lib/common/htmlparse.y"
                           { 
           if (nonSpace(agxbuse(HTMLstate.str))) {
             htmlerror ("Syntax error: non-space string used before <TABLE>");
@@ -2040,11 +2026,11 @@ yyreduce:
           (yyvsp[0].tbl)->font = HTMLstate.fontstack->cfont;
           (yyval.tbl) = (yyvsp[0].tbl);
         }
-#line 2044 "htmlparse.c"
+#line 2030 "htmlparse.c"
     break;
 
   case 39:
-#line 513 "../../lib/common/htmlparse.y"
+#line 501 "../../lib/common/htmlparse.y"
                                    {
           if (nonSpace(agxbuse(HTMLstate.str))) {
             htmlerror ("Syntax error: non-space string used after </TABLE>");
@@ -2053,155 +2039,155 @@ yyreduce:
           (yyval.tbl) = HTMLstate.tblstack;
           HTMLstate.tblstack = HTMLstate.tblstack->u.p.prev;
         }
-#line 2057 "htmlparse.c"
+#line 2043 "htmlparse.c"
     break;
 
   case 40:
-#line 523 "../../lib/common/htmlparse.y"
+#line 511 "../../lib/common/htmlparse.y"
                   { (yyval.tbl) = (yyvsp[0].tbl); }
-#line 2063 "htmlparse.c"
+#line 2049 "htmlparse.c"
     break;
 
   case 41:
-#line 524 "../../lib/common/htmlparse.y"
+#line 512 "../../lib/common/htmlparse.y"
                               { (yyval.tbl)=(yyvsp[-1].tbl); }
-#line 2069 "htmlparse.c"
+#line 2055 "htmlparse.c"
     break;
 
   case 42:
-#line 525 "../../lib/common/htmlparse.y"
+#line 513 "../../lib/common/htmlparse.y"
                                   { (yyval.tbl)=(yyvsp[-1].tbl); }
-#line 2075 "htmlparse.c"
+#line 2061 "htmlparse.c"
     break;
 
   case 43:
-#line 526 "../../lib/common/htmlparse.y"
+#line 514 "../../lib/common/htmlparse.y"
                                         { (yyval.tbl)=(yyvsp[-1].tbl); }
-#line 2081 "htmlparse.c"
+#line 2067 "htmlparse.c"
     break;
 
   case 44:
-#line 527 "../../lib/common/htmlparse.y"
+#line 515 "../../lib/common/htmlparse.y"
                                       { (yyval.tbl)=(yyvsp[-1].tbl); }
-#line 2087 "htmlparse.c"
+#line 2073 "htmlparse.c"
     break;
 
   case 45:
-#line 528 "../../lib/common/htmlparse.y"
+#line 516 "../../lib/common/htmlparse.y"
                               { (yyval.tbl)=(yyvsp[-1].tbl); }
-#line 2093 "htmlparse.c"
+#line 2079 "htmlparse.c"
     break;
 
   case 48:
-#line 535 "../../lib/common/htmlparse.y"
+#line 523 "../../lib/common/htmlparse.y"
            { (yyval.p) = (yyvsp[0].p); }
-#line 2099 "htmlparse.c"
+#line 2085 "htmlparse.c"
     break;
 
   case 49:
-#line 536 "../../lib/common/htmlparse.y"
+#line 524 "../../lib/common/htmlparse.y"
                 { (yyval.p) = (yyvsp[0].p); }
-#line 2105 "htmlparse.c"
+#line 2091 "htmlparse.c"
     break;
 
   case 50:
-#line 537 "../../lib/common/htmlparse.y"
+#line 525 "../../lib/common/htmlparse.y"
                    { (yyvsp[-2].p)->ruled = 1; (yyval.p) = (yyvsp[0].p); }
-#line 2111 "htmlparse.c"
+#line 2097 "htmlparse.c"
     break;
 
   case 51:
-#line 540 "../../lib/common/htmlparse.y"
+#line 528 "../../lib/common/htmlparse.y"
             { addRow (); }
-#line 2117 "htmlparse.c"
+#line 2103 "htmlparse.c"
     break;
 
   case 52:
-#line 540 "../../lib/common/htmlparse.y"
+#line 528 "../../lib/common/htmlparse.y"
                                            { (yyval.p) = lastRow(); }
-#line 2123 "htmlparse.c"
+#line 2109 "htmlparse.c"
     break;
 
   case 53:
-#line 543 "../../lib/common/htmlparse.y"
+#line 531 "../../lib/common/htmlparse.y"
              { (yyval.cell) = (yyvsp[0].cell); }
-#line 2129 "htmlparse.c"
+#line 2115 "htmlparse.c"
     break;
 
   case 54:
-#line 544 "../../lib/common/htmlparse.y"
+#line 532 "../../lib/common/htmlparse.y"
                    { (yyval.cell) = (yyvsp[0].cell); }
-#line 2135 "htmlparse.c"
+#line 2121 "htmlparse.c"
     break;
 
   case 55:
-#line 545 "../../lib/common/htmlparse.y"
+#line 533 "../../lib/common/htmlparse.y"
                       { (yyvsp[-2].cell)->ruled |= HTML_VRULE; (yyval.cell) = (yyvsp[0].cell); }
-#line 2141 "htmlparse.c"
+#line 2127 "htmlparse.c"
     break;
 
   case 56:
-#line 548 "../../lib/common/htmlparse.y"
+#line 536 "../../lib/common/htmlparse.y"
                         { setCell((yyvsp[-1].cell),(yyvsp[0].tbl),HTML_TBL); }
-#line 2147 "htmlparse.c"
+#line 2133 "htmlparse.c"
     break;
 
   case 57:
-#line 548 "../../lib/common/htmlparse.y"
+#line 536 "../../lib/common/htmlparse.y"
                                                                 { (yyval.cell) = (yyvsp[-3].cell); }
-#line 2153 "htmlparse.c"
+#line 2139 "htmlparse.c"
     break;
 
   case 58:
-#line 549 "../../lib/common/htmlparse.y"
+#line 537 "../../lib/common/htmlparse.y"
                        { setCell((yyvsp[-1].cell),(yyvsp[0].txt),HTML_TEXT); }
-#line 2159 "htmlparse.c"
+#line 2145 "htmlparse.c"
     break;
 
   case 59:
-#line 549 "../../lib/common/htmlparse.y"
+#line 537 "../../lib/common/htmlparse.y"
                                                                 { (yyval.cell) = (yyvsp[-3].cell); }
-#line 2165 "htmlparse.c"
+#line 2151 "htmlparse.c"
     break;
 
   case 60:
-#line 550 "../../lib/common/htmlparse.y"
+#line 538 "../../lib/common/htmlparse.y"
                     { setCell((yyvsp[-1].cell),(yyvsp[0].img),HTML_IMAGE); }
-#line 2171 "htmlparse.c"
+#line 2157 "htmlparse.c"
     break;
 
   case 61:
-#line 550 "../../lib/common/htmlparse.y"
+#line 538 "../../lib/common/htmlparse.y"
                                                               { (yyval.cell) = (yyvsp[-3].cell); }
-#line 2177 "htmlparse.c"
+#line 2163 "htmlparse.c"
     break;
 
   case 62:
-#line 551 "../../lib/common/htmlparse.y"
+#line 539 "../../lib/common/htmlparse.y"
               { setCell((yyvsp[0].cell),mkText(),HTML_TEXT); }
-#line 2183 "htmlparse.c"
+#line 2169 "htmlparse.c"
     break;
 
   case 63:
-#line 551 "../../lib/common/htmlparse.y"
+#line 539 "../../lib/common/htmlparse.y"
                                                              { (yyval.cell) = (yyvsp[-2].cell); }
-#line 2189 "htmlparse.c"
+#line 2175 "htmlparse.c"
     break;
 
   case 64:
-#line 554 "../../lib/common/htmlparse.y"
+#line 542 "../../lib/common/htmlparse.y"
                          { (yyval.img) = (yyvsp[-1].img); }
-#line 2195 "htmlparse.c"
+#line 2181 "htmlparse.c"
     break;
 
   case 65:
-#line 555 "../../lib/common/htmlparse.y"
+#line 543 "../../lib/common/htmlparse.y"
                { (yyval.img) = (yyvsp[0].img); }
-#line 2201 "htmlparse.c"
+#line 2187 "htmlparse.c"
     break;
 
 
-#line 2205 "htmlparse.c"
+#line 2191 "htmlparse.c"
 
       default: break;
     }
@@ -2433,7 +2419,7 @@ yyreturn:
 #endif
   return yyresult;
 }
-#line 567 "../../lib/common/htmlparse.y"
+#line 555 "../../lib/common/htmlparse.y"
 
 
 /* parseHTML:
