@@ -223,13 +223,13 @@ extern int ex_debug;
 #if ! defined EX_STYPE && ! defined EX_STYPE_IS_DECLARED
 union EX_STYPE
 {
-#line 39 "../../lib/expr/exparse.y"
+#line 40 "../../lib/expr/exparse.y"
 
 	struct Exnode_s*expr;
 	double		floating;
 	struct Exref_s*	reference;
 	struct Exid_s*	id;
-	Sflong_t	integer;
+	long long integer;
 	int		op;
 	char*		string;
 
